@@ -30,7 +30,7 @@ let stringset_of_list pkgs =
 
 let fedora_detect () =
   Config.rpm <> "no" && Config.rpm2cpio <> "no" && rpm_is_available () &&
-    Config.yumdownloader <> "no" &&
+    (Config.yumdownloader <> "no" || Config.dnf <> "no") &&
     try
       (stat "/etc/redhat-release").st_kind = S_REG ||
       (stat "/etc/fedora-release").st_kind = S_REG
@@ -47,6 +47,13 @@ let mageia_detect () =
     Config.fakeroot <> "no" &&
     try (stat "/etc/mageia-release").st_kind = S_REG with Unix_error _ -> false
 
+let ibm_powerkvm_detect () =
+  Config.rpm <> "no" && Config.rpm2cpio <> "no" && rpm_is_available () &&
+    Config.yumdownloader <> "no" &&
+    try
+      (stat "/etc/ibm_powerkvm-release").st_kind = S_REG
+    with Unix_error _ -> false
+
 let settings = ref no_settings
 let rpm_major, rpm_minor = ref 0, ref 0
 let zypper_major, zypper_minor, zypper_patch = ref 0, ref 0, ref 0
@@ -144,7 +151,7 @@ let rpm_package_of_string str =
      * architecture.
      *)
     let cmp { version = v1; arch = a1 } { version = v2; arch = a2 } =
-      let i = compare_version v2 v1 in
+      let i = rpm_vercmp v2 v1 in
       if i <> 0 then i
       else compare_architecture a2 a1
     in
@@ -203,8 +210,51 @@ let rpm_package_name pkg =
 let rpm_get_package_database_mtime () =
   (lstat "/var/lib/rpm/Packages").st_mtime
 
-(* Memo of resolved provides. *)
-let rpm_providers = Hashtbl.create 13
+(* Return the best provider of a particular RPM requirement.
+ *
+ * There may be multiple, or no providers.  In case there are multiple,
+ * choose the one with the shortest name (as yum used to).
+ *
+ * We could do better here: http://yum.baseurl.org/wiki/CompareProviders
+ *)
+let provider =
+  (* Memo of resolved provides. *)
+  let rpm_providers = Hashtbl.create 13 in
+  fun req ->
+    try Hashtbl.find rpm_providers req
+    with Not_found ->
+      let ret =
+        try
+          (* 'providers' here is an array of just package names. *)
+          let providers = rpm_pkg_whatprovides (get_rpm ()) req in
+          let providers = Array.to_list providers in
+          (* --whatprovides will return duplicate identical packages, so: *)
+          let providers = sort_uniq providers in
+          (* Only packages which are actually installed: *)
+          let providers =
+            List.filter (fun name -> rpm_package_of_string name <> None)
+                        providers in
+
+          match providers with
+          | [] -> None
+          | [name] ->
+             Some name
+          | names ->
+             if !settings.debug >= 2 then
+               printf "supermin: rpm: multiple providers: requirement %s: providers: %s\n"
+                      req (String.concat " " names);
+             let cmp name1 name2 =
+               let len1 = String.length name1 and len2 = String.length name2 in
+               if len1 <> len2 then compare len1 len2
+               else compare name1 name2 in
+             let names = List.sort cmp names in
+             let best_name = List.hd names in
+             if !settings.debug >= 2 then
+               printf "supermin: rpm: multiple providers: picked %s\n" best_name;
+             Some best_name
+        with Not_found -> None in
+      Hashtbl.add rpm_providers req ret;
+      ret
 
 let rpm_get_all_requires pkgs =
   let get pkg =
@@ -219,20 +269,9 @@ let rpm_get_all_requires pkgs =
               rpm_pkg_requires (get_rpm ()) (rpm_package_to_string pkg) in
     let pkgs' = Array.fold_left (
       fun set x ->
-        try
-          let provides =
-            try Hashtbl.find rpm_providers x
-            with Not_found ->
-              let p = rpm_pkg_whatprovides (get_rpm ()) x in
-              Hashtbl.add rpm_providers x p;
-              p in
-          Array.fold_left (
-            fun newset p ->
-              match rpm_package_of_string p with
-                | None -> newset
-                | Some x -> StringSet.add p newset
-          ) set provides
-        with Not_found -> set
+        match provider x with
+        | None -> set
+        | Some p -> StringSet.add p set
     ) StringSet.empty reqs in
     pkgs'
   in
@@ -278,20 +317,44 @@ let rpm_get_all_files pkgs =
 let rec fedora_download_all_packages pkgs dir =
   let tdir = !settings.tmpdir // string_random8 () in
 
+  if Config.dnf <> "no" then
+    fedora_download_all_packages_with_dnf pkgs dir tdir
+  else (* Config.yumdownloader <> "no" *)
+    fedora_download_all_packages_with_yum pkgs dir tdir;
+
+  rpm_unpack tdir dir
+
+and fedora_download_all_packages_with_dnf pkgs dir tdir =
+  (* dnf doesn't create the download directory, and if you use
+   * --destdir without an existing directory then it downloads each
+   * package on top of each other to --destdir as a file.  WTF?
+   *)
+  mkdir tdir 0o700;
+
+  let rpms = pkgs_as_NA_rpms pkgs in
+
+  let cmd =
+    sprintf "%s download%s%s --destdir %s %s"
+      Config.dnf
+      (if !settings.debug >= 1 then " -v" else " -q")
+      (match !settings.packager_config with
+      | None -> ""
+      | Some filename -> sprintf " -c %s" (quote filename))
+      (quote tdir)
+      (quoted_list rpms) in
+  run_command cmd
+
+and fedora_download_all_packages_with_yum pkgs dir tdir =
   (* It's quite complex to get yumdownloader to download specific
    * RPMs.  If we use the full NVR, then it will refuse if an installed
    * RPM is older than whatever is currently in the repo.  If we use
    * just name, it will download all architectures (even with
    * --archlist).
-   * 
+   *
    * Use name.arch so it can download any version but only the specific
    * architecture.
    *)
-  let rpms = List.map rpm_of_pkg (PackageSet.elements pkgs) in
-  let rpms = List.map (
-    fun { name = name; arch = arch } ->
-      sprintf "%s.%s" name arch
-  ) rpms in
+  let rpms = pkgs_as_NA_rpms pkgs in
 
   let cmd =
     sprintf "%s%s%s --destdir %s %s"
@@ -302,18 +365,12 @@ let rec fedora_download_all_packages pkgs dir =
       | Some filename -> sprintf " -c %s" (quote filename))
       (quote tdir)
       (quoted_list rpms) in
-  run_command cmd;
-
-  rpm_unpack tdir dir
+  run_command cmd
 
 and opensuse_download_all_packages pkgs dir =
   let tdir = !settings.tmpdir // string_random8 () in
 
-  let rpms = List.map rpm_of_pkg (PackageSet.elements pkgs) in
-  let rpms = List.map (
-    fun { name = name; arch = arch } ->
-      sprintf "%s.%s" name arch
-  ) rpms in
+  let rpms = pkgs_as_NA_rpms pkgs in
 
   let is_zypper_1_9_14 =
     !zypper_major > 1
@@ -382,6 +439,13 @@ and mageia_download_all_packages pkgs dir =
 
   rpm_unpack tdir dir
 
+and pkgs_as_NA_rpms pkgs =
+  let rpms = List.map rpm_of_pkg (PackageSet.elements pkgs) in
+  List.map (
+    fun { name = name; arch = arch } ->
+      sprintf "%s.%s" name arch
+  ) rpms
+
 and rpm_unpack tdir dir =
   (* Unpack each downloaded package.
    * 
@@ -412,6 +476,11 @@ let () =
     ph_download_package = PHDownloadAllPackages fedora_download_all_packages;
   } in
   register_package_handler "fedora" "rpm" fedora;
+  let ibm_powerkvm = {
+    fedora with
+    ph_detect = ibm_powerkvm_detect;
+  } in
+  register_package_handler "ibm_powerkvm" "rpm" ibm_powerkvm;
   let opensuse = {
     fedora with
     ph_detect = opensuse_detect;
